// -*-c++-*-
/* $Id: dup.h,v 1.7 1999/07/17 16:39:50 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#ifndef _DUP_H_
#define _DUP_H_ 1

#include "amisc.h"
#include "itree.h"

u_int64_t polymod (u_int64_t nh, u_int64_t nl, u_int64_t d);
void polymult (u_int64_t *php, u_int64_t *plp, u_int64_t x, u_int64_t y);
bool polyirreducible (u_int64_t f);
u_int64_t polygen (u_int degree);

class rabinpoly {
  int shift;
  u_int64_t T[256];		// Lookup table for mod
  void calcT ();
public:
  const u_int64_t poly;		// Actual polynomial

  explicit rabinpoly (u_int64_t poly);
  u_int64_t append8 (u_int64_t p, u_char m) const
    { return ((p << 8) | m) ^ T[p >> shift]; }
};

class sketch {
public:
  enum { samplesize = 128 };
  enum { max_resemblance = 0x100000 };
  static const u_int64_t noshingle;
  typedef u_int64_t buf[samplesize];
private:
  enum { shinglesize = 6 };

  const rabinpoly &poly;
  bool inword;
  u_int wordno;
  u_int64_t prints[shinglesize];

  struct shingle {
    const u_int64_t val;
    itree_entry<shingle> link;
    shingle (u_int64_t v) : val (v) {}
  };
  typedef itree<const u_int64_t, shingle, &shingle::val, &shingle::link> stree;
  u_int nshingles;
  u_int64_t maxshingle;
  stree shingles;

  void addshingle (u_int64_t s);
  void delshingle (shingle *sp);
public:
  sketch (const rabinpoly &poly);
  ~sketch ();
  void update (const void *data, size_t len);
  void extract (buf out) const;
  static u_int32_t resemblance (buf s1, buf s2);
};

int sketchfd (const rabinpoly &poly, int fd, sketch::buf out);
int sketchfile (const rabinpoly &poly, const char *path, sketch::buf out);
str sketch822 (const rabinpoly &poly, int fd, sketch::buf b);

struct dupdb {
  enum { magic_val = 0x19b9482d };

  struct entry {
    u_int32_t dupcount;
    u_int32_t entrysize;
    u_int32_t shingles[sketch::samplesize];

    void init (const sketch::buf);
    u_int32_t compare (const entry &e) const;
  };

  u_int64_t magic;
  u_int64_t poly;
  u_int64_t nentries;
  u_int32_t threshold;
  u_int32_t __spare;
  entry entries[1];

  static size_t dbsize (size_t nelm)
    { return offsetof (dupdb, entries[nelm]); }

  void init (u_int64_t poly, u_int threshold_percent = 0);
  size_t size () const { return dbsize (nentries); }
  bool check (size_t len) const;
  entry *lookup (const entry &e);
};

class dupfile {
  char *const path;
  dupdb *dbp;
  size_t dbsize;
  int fd;
  const bool update;
  bool created;
  rabinpoly *rp;
  void lock (bool shared = false);
  void unlock ();
  void unmap ();
  void remap ();
public:
  dupfile (const char *p, bool u = true)
    : path (xstrdup (p)), dbp (NULL), dbsize (0),
      fd (-1), update (u), created (false), rp (NULL) {}
  ~dupfile ();
  bool init (u_int threshold_percent = 0);
  ssize_t lookup (const sketch::buf b);
  u_int32_t changecount (size_t entryno, int32_t inc);
  const rabinpoly &getpoly () const { assert (fd >= 0); return *rp; }
  bool isnew () const { return created; }
  const dupdb *getdb () const { return dbp; }
};

#endif /* !_DUP_H_ */
