/* $Id: dupdetect.C,v 1.7 1999/07/17 16:18:15 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "dup.h"
#include "parseopt.h"
#include "db.h"

class msgdb {
  const str path;
  DB *db;
  int fd;
public:
  msgdb (const str &p) : path (p), db (NULL), fd (-1) {}
  ~msgdb () { close (); }
  bool open (bool trunc = false);
  void close ();
  bool check (str id);
  bool insert (str id);
  void remove (str id);
};

void
msgdb::close ()
{
  if (db) {
    db->close (db, 0);
    db = NULL;
    fd = -1;
  }
}

bool
msgdb::open (bool trunc)
{
  if (!db_open (path.cstr (), DB_HASH,
		DB_CREATE | (trunc ? DB_TRUNCATE : 0),
		0666, NULL, NULL, &db)
      && !db->fd (db, &fd)
      && !flock (fd, LOCK_EX))
    return true;
  close ();
  return false;
}

bool
msgdb::check (str id)
{
  DBT key, dat;
  bzero (&key, sizeof (key));
  key.data = const_cast<char *> (id.cstr ());
  key.size = id.len ();
  return !db->get (db, NULL, &key, &dat, 0);
}

bool
msgdb::insert (str id)
{
  static DBT nulldat;
  DBT key;
  bzero (&key, sizeof (key));
  key.data = const_cast<char *> (id.cstr ());
  key.size = id.len ();
  return !db->put (db, NULL, &key, &nulldat, DB_NOOVERWRITE);
}

void
msgdb::remove (str id)
{
  DBT key;
  bzero (&key, sizeof (key));
  key.data = const_cast<char *> (id.cstr ());
  key.size = id.len ();
  db->del (db, NULL, &key, 0);
}

static void
exit75 ()
{
  exit (75);
}

int
main (int argc, char **argv)
{
  errfd = 1;
  fatalhook = exit75;

  int count;
  if (argc != 3 || !convertint (argv[2], &count))
    fatal ("usage: dupdetect dbfile count\n");
  setprogname (argv[0]);

  static str skfile = strbuf ("%s.sk", argv[1]);
  static str idfile = strbuf ("%s.id", argv[1]);

  dupfile d (skfile);
  if (!d.init (85))
    fatal ("%s: could not initialize database\n", argv[1]);
  sketch::buf b;
  str msgid = sketch822 (d.getpoly (), 0, b);
  ssize_t n = d.lookup (b);

  msgdb mdb (idfile);
  if (!mdb.open (d.isnew ()) && (d.isnew () || !mdb.open (true)))
    fatal ("could not open Message-ID database\n");
  bool newmsg = count <= 0 || mdb.insert (msgid);
  if (count < 0)
    mdb.remove (msgid);

  int bi = 0;
  if (n >= 0) {
    int delta = int (count >= 0 ? 100 * sqrt (count) : -100 * sqrt (-count));
    bi = d.changecount (n, newmsg ? delta : 0);
    if (delta > 0) {
      if (bi >= 2000) {
	d.changecount (n, -delta);
	mdb.remove (msgid);
	warnx ("duplicate (BI = %d.%.2d)\n", bi / 100, bi % 100);
	return 65;
      }
      else if (bi >= 500) {
	warnx ("duplicate (BI = %d.%.2d)\n", bi / 100, bi % 100);
	return 75;
      }
    }
  }

  warnx ("BI = %d.%.2d\n", bi / 100, bi % 100);
  return 0;
}
