/* $Id: duplookup.C,v 1.3 1999/07/17 16:39:50 dm Exp $ */

/*
 *
 * Copyright (C) 1999 David Mazieres (dm@uun.org)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2, or (at
 * your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 *
 */

#include "dup.h"
#include "parseopt.h"
#include "rxx.h"
#include "db.h"


int
main (int argc, char **argv)
{
  if (argc < 2)
    fatal ("usage: dupdetect dbfile [file ...]\n");
  setprogname (argv[0]);

  static str skfile = strbuf ("%s.sk", argv[1]);

  dupfile d (skfile, false);
  if (!d.init (85))
    fatal ("%s: could not initialize database\n", argv[1]);

  sketch::buf b;
  dupdb::entry e;

  if (argc == 2) {
    sketch822 (d.getpoly (), 0, b);
    e.init (b);
    ssize_t n = d.lookup (b);
    double sim = n < 0 ? 0.0
      : e.compare (d.getdb ()->entries[n]) / double (sketch::max_resemblance);
    printf ("%ld %f\n", long (d.lookup (b)), sim);
    return 0;
  }

  for (int i = 2; i < argc; i++) {
    int fd = open (argv[i], O_RDONLY);
    if (fd < 0)
      warn ("%s: %m\n", argv[i]);
    else {
      sketch822 (d.getpoly (), fd, b);
      e.init (b);
      ssize_t n = d.lookup (b);
      double sim = n < 0 ? 0.0
	: (e.compare (d.getdb ()->entries[n])
	   / double (sketch::max_resemblance));
      close (fd);
      printf ("%s: %ld %f\n", argv[i], long (d.lookup (b)), sim);
    }
  }

  return 0;
}
